@@ -115,9 +115,6 @@ class DiagramTreeBuilder:
             elif isinstance(stmt, diagparser.DefAttrs):
                 group.set_attributes(stmt.attrs)
 
-            else:
-                raise AttributeError("Unknown sentense: " + str(type(stmt)))
-
         group.update_order()
         return group
 
@@ -468,35 +465,146 @@ class DiagramLayoutManager:
             return None
 
 
+class EdgeLayoutManager(object):
+    def __init__(self, diagram):
+        self.diagram = diagram
+
+    @property
+    def groups(self):
+        if self.diagram.separated:
+            seq = self.diagram.nodes
+        else:
+            seq = self.diagram.traverse_groups(preorder=True)
+
+        for group in seq:
+            if not group.drawable:
+                yield group
+
+    @property
+    def nodes(self):
+        if self.diagram.separated:
+            seq = self.diagram.nodes
+        else:
+            seq = self.diagram.traverse_nodes()
+
+        for node in seq:
+            if node.drawable:
+                yield node
+
+    @property
+    def edges(self):
+        for edge in (e for e in self.diagram.edges  if e.style != 'none'):
+            yield edge
+
+        for group in self.groups:
+            for edge in (e for e in group.edges  if e.style != 'none'):
+                yield edge
+
+    def run(self):
+        for edge in self.edges:
+            dir = edge.direction
+
+            if edge.node1.group.orientation == 'landscape':
+                if dir == 'right':
+                    r = range(edge.node1.xy.x + 1, edge.node2.xy.x)
+                    for x in r:
+                        xy = (x, edge.node1.xy.y)
+                        nodes = [x for x in self.nodes if x.xy == xy]
+                        if len(nodes) > 0:
+                            edge.skipped = 1
+                elif dir == 'right-up':
+                    r = range(edge.node1.xy.x + 1, edge.node2.xy.x)
+                    for x in r:
+                        xy = (x, edge.node1.xy.y)
+                        nodes = [x for x in self.nodes if x.xy == xy]
+                        if len(nodes) > 0:
+                            edge.skipped = 1
+                elif dir == 'right-down':
+                    if self.diagram.edge_layout == 'flowchart':
+                        r = range(edge.node1.xy.y, edge.node2.xy.y)
+                        for y in r:
+                            xy = (edge.node1.xy.x, y + 1)
+                            nodes = [x for x in self.nodes if x.xy == xy]
+                            if len(nodes) > 0:
+                                edge.skipped = 1
+
+                    r = range(edge.node1.xy.x + 1, edge.node2.xy.x)
+                    for x in r:
+                        xy = (x, edge.node2.xy.y)
+                        nodes = [x for x in self.nodes if x.xy == xy]
+                        if len(nodes) > 0:
+                            edge.skipped = 1
+                elif dir in ('left-down', 'down'):
+                    r = range(edge.node1.xy.y + 1, edge.node2.xy.y)
+                    for y in r:
+                        xy = (edge.node1.xy.x, y)
+                        nodes = [x for x in self.nodes if x.xy == xy]
+                        if len(nodes) > 0:
+                            edge.skipped = 1
+                elif dir == 'up':
+                    r = range(edge.node2.xy.y + 1, edge.node1.xy.y)
+                    for y in r:
+                        xy = (edge.node1.xy.x, y)
+                        nodes = [x for x in self.nodes if x.xy == xy]
+                        if len(nodes) > 0:
+                            edge.skipped = 1
+            else:
+                if dir == 'right':
+                    r = range(edge.node1.xy.x + 1, edge.node2.xy.x)
+                    for x in r:
+                        xy = (x, edge.node1.xy.y)
+                        nodes = [x for x in self.nodes if x.xy == xy]
+                        if len(nodes) > 0:
+                            edge.skipped = 1
+                elif dir in ('left-down', 'down'):
+                    r = range(edge.node1.xy.y + 1, edge.node2.xy.y)
+                    for y in r:
+                        xy = (edge.node1.xy.x, y)
+                        nodes = [x for x in self.nodes if x.xy == xy]
+                        if len(nodes) > 0:
+                            edge.skipped = 1
+                elif dir == 'right-down':
+                    if self.diagram.edge_layout == 'flowchart':
+                        r = range(edge.node1.xy.x, edge.node2.xy.x)
+                        for x in r:
+                            xy = (x + 1, edge.node1.xy.y)
+                            nodes = [x for x in self.nodes if x.xy == xy]
+                            if len(nodes) > 0:
+                                edge.skipped = 1
+
+                    r = range(edge.node1.xy.y + 1, edge.node2.xy.y)
+                    for y in r:
+                        xy = (edge.node2.xy.x, y)
+                        nodes = [x for x in self.nodes if x.xy == xy]
+                        if len(nodes) > 0:
+                            edge.skipped = 1
+
+
 class ScreenNodeBuilder:
     @classmethod
-    def build(klass, tree, layout=True):
+    def build(cls, tree, layout=True):
         DiagramNode.clear()
         DiagramEdge.clear()
         NodeGroup.clear()
         Diagram.clear()
 
-        diagram = DiagramTreeBuilder().build(tree)
-        if layout:
-            DiagramLayoutManager(diagram).run()
-            diagram.fixiate(True)
+        return cls(tree, layout).run()
 
-        return diagram
-
-
-class SeparateDiagramBuilder:
-    @classmethod
-    def build(cls, tree):
-        DiagramNode.clear()
-        DiagramEdge.clear()
-        NodeGroup.clear()
-        Diagram.clear()
-
-        return cls(tree).run()
-
-    def __init__(self, tree):
+    def __init__(self, tree, layout):
         self.diagram = DiagramTreeBuilder().build(tree)
+        self.layout = layout
 
+    def run(self):
+        if self.layout:
+            DiagramLayoutManager(self.diagram).run()
+            self.diagram.fixiate(True)
+
+        EdgeLayoutManager(self.diagram).run()
+
+        return self.diagram
+
+
+class SeparateDiagramBuilder(ScreenNodeBuilder):
     @property
     def _groups(self):
         # Store nodes and edges of subgroups
@@ -529,6 +637,7 @@ class SeparateDiagramBuilder:
                 n.xy = XY(0, 0)
                 n.width = 1
                 n.height = 1
+                n.separated = False
 
             for edge in DiagramEdge.find_all():
                 edge.skipped = False
@@ -588,6 +697,7 @@ class SeparateDiagramBuilder:
                 if isinstance(g, NodeGroup):
                     g.nodes = []
                     g.edges = []
+                    g.separated = True
 
             # pick up nodes to base diagram
             nodes1 = [e.node1 for e in DiagramEdge.find(None, group)]
@@ -607,5 +717,6 @@ class SeparateDiagramBuilder:
 
             DiagramLayoutManager(base).run()
             base.fixiate(True)
+            EdgeLayoutManager(base).run()
 
             yield base
