@@ -167,7 +167,7 @@ class DiagramNode(Element):
             self.style = value
         else:
             msg = "WARNING: unknown node style: %s\n" % value
-            sys.stderr.write(msg)
+            raise AttributeError(msg)
 
     def set_shape(self, value):
         try:
@@ -175,7 +175,7 @@ class DiagramNode(Element):
             self.shape = value
         except:
             msg = "WARNING: unknown node shape: %s\n" % value
-            raise RuntimeError(msg)
+            raise AttributeError(msg)
 
     def set_icon(self, value):
         if urlutil.isurl(value) or os.path.isfile(value):
@@ -206,12 +206,12 @@ class NodeGroup(Element):
     def __init__(self, id):
         super(NodeGroup, self).__init__(id)
 
-        self.href = None
         self.level = 0
         self.separated = False
         self.shape = 'box'
         self.nodes = []
         self.edges = []
+        self.icon = None
         self.orientation = 'landscape'
 
     def duplicate(self):
@@ -221,18 +221,15 @@ class NodeGroup(Element):
 
         return copied
 
-    def parent(self, level=None):
-        if level is None:
-            return self.group
-        else:
-            if self.level < level:
-                return None
+    def parent(self, level):
+        if self.level < level:
+            return None
 
-            group = self
-            while group.level != level:
-                group = group.group
+        group = self
+        while group.level != level:
+            group = group.group
 
-            return group
+        return group
 
     def is_parent(self, other):
         parent = self.parent(other.level)
@@ -285,7 +282,7 @@ class NodeGroup(Element):
             self.orientation = value
         else:
             msg = "WARNING: unknown diagram orientation: %s\n" % value
-            sys.stderr.write(msg)
+            raise AttributeError(msg)
 
 
 class DiagramEdge(Base):
@@ -406,7 +403,7 @@ class DiagramEdge(Base):
             self.dir = 'none'
         else:
             msg = "WARNING: unknown edge dir: %s\n" % value
-            sys.stderr.write(msg)
+            raise AttributeError(msg)
 
     def set_color(self, color):
         self.color = images.color_to_rgb(color)
@@ -417,7 +414,7 @@ class DiagramEdge(Base):
             self.style = value
         else:
             msg = "WARNING: unknown edge style: %s\n" % value
-            sys.stderr.write(msg)
+            raise AttributeError(msg)
 
     def set_hstyle(self, value):
         value = value.lower()
@@ -425,7 +422,7 @@ class DiagramEdge(Base):
             self.hstyle = value
         else:
             msg = "WARNING: unknown edge hstyle: %s\n" % value
-            sys.stderr.write(msg)
+            raise AttributeError(msg)
 
     def set_folded(self, value):
         self.folded = True
@@ -433,6 +430,35 @@ class DiagramEdge(Base):
     def set_nofolded(self, value):
         self.folded = False
 
+    @property
+    def direction(self):
+        node1 = self.node1.xy
+        node2 = self.node2.xy
+
+        if node1.x > node2.x:
+            if node1.y > node2.y:
+                dir = 'left-up'
+            elif node1.y == node2.y:
+                dir = 'left'
+            else:
+                dir = 'left-down'
+        elif node1.x == node2.x:
+            if node1.y > node2.y:
+                dir = 'up'
+            elif node1.y == node2.y:
+                dir = 'same'
+            else:
+                dir = 'down'
+        else:
+            if node1.y > node2.y:
+                dir = 'right-up'
+            elif node1.y == node2.y:
+                dir = 'right'
+            else:
+                dir = 'right-down'
+
+        return dir
+
 
 class Diagram(NodeGroup):
     _DiagramNode = DiagramNode
@@ -464,7 +490,7 @@ class Diagram(NodeGroup):
             DiagramNode.set_default_shape(value)
         except:
             msg = "WARNING: unknown node shape: %s\n" % value
-            raise RuntimeError(msg)
+            raise AttributeError(msg)
 
     def set_default_text_color(self, color):
         self.textcolor = images.color_to_rgb(color)
@@ -495,5 +521,5 @@ class Diagram(NodeGroup):
 
             self.edge_layout = value
         else:
-            msg = "WARNING: unknown edge dir: %s\n" % value
-            sys.stderr.write(msg)
+            msg = "WARNING: unknown edge layout: %s\n" % value
+            raise AttributeError(msg)
